@@ -344,6 +344,7 @@ cum2 <- Cpred(object$cum,times);
 if (!is.null(times2)) cum2 <- Cpred(object$cum,times2);
 
 if (is.null(X)) X <- 1;
+if (is.null(Z) && (!is.null(object$gamma))) Z <- matrix(0,1,nrow(object$gamma));
 if (is.null(X) & (!is.null(Z))) { Z <- as.matrix(Z);  X <- matrix(1,nrow(Z),1)}
 if (is.null(Z) & (!is.null(X)))  {X <- as.matrix(X);  Z <- matrix(0,nrow(X),1); gamma <- 0}
 
@@ -359,18 +360,20 @@ if (diag==FALSE) {
 }
 
 if (!is.null(object$gamma)) {
-	RR<- exp(Z%*%gamma); 
-	cumhaz <- t( t(time.part) * RR );  
-	cumhaz2 <- t( t(time.part2) * RR )} else {
-		cumhaz <- time.part;  cumhaz2 <- time.part2; 
+	RR<- exp(Z%*%object$gamma); 
+	cumhaz <- (time.part) * RR ;  
+	cumhaz2 <- (time.part2) * RR; 
+} else {
+	cumhaz <- time.part;  cumhaz2 <- time.part2; 
 } 
 S1 <- exp(-cumhaz); S2 <- exp(-cumhaz2)
 
 if (attr(object,"var.link")==1) theta  <- exp(object$theta) else theta <- object$theta
 if (!is.null(theta.des)) theta <- c(theta.des %*% object$theta)
 
-if (diag==FALSE) St1t2<- (outer(c(S1)^{-(1/theta)},c(S2)^{-(1/theta)},FUN="+") - 1)^(-(theta)) else 
-St1t2<- ((S1^{-(1/theta)}+S2^{-(1/theta)})-1)^(-(theta))
+if (diag==FALSE) St1t2<- (outer(c(S1)^{-(theta)},c(S2)^{-(theta)},FUN="+") - 1)^(-(1/theta)) else 
+St1t2<- ((S1^{-(theta)}+S2^{-(theta)})-1)^(-(1/theta))
+###St1t2<- ((S1^{-(1/theta)}+S2^{-(1/theta)})-1)^(-(theta))
 
 out=list(St1t2=St1t2,S1=S1,S2=S2,times=times,times2=times2,theta=theta)
 return(out)
