@@ -190,11 +190,16 @@ vector *extract_col(matrix *M, int col_to_get, vector *v);
 
 void Cpred(double *cum,int *nx,int *px,double *xval,int *nxval,double *pred); 
 
-void sindex(int *index, double *jump, double *eval, int *njump, int *neval);
+void sindex(int *index, double *jump, double *eval, int *njump, int *neval,int *strict);
 
 void nclusters(int *npers,int *clusters, int *nclust, int *mclust);
 
-void clusterindex(int *clusters, int *nclust,int *npers,int *idclust, int *clustsize,int *mednum,int *num); 
+void clusterindex(int *clusters, int *nclust,int *npers,int *idclust, int *clustsize,
+	          int *mednum,int *num,int *firstclustid); 
+
+
+void clusterindexdata(int *clusters,int *nclust,int *npers,int *idclust,int *clustsize,int *mednum,
+		int *num,double *data, int *p,double *nydata); 
  
 void comptest(double *times,int *Ntimes,int *px,double *cu,double *vcu,
 	      double *vcudif,int *antsim,double *test,double *testOBS,
@@ -217,7 +222,7 @@ void itfitsemi(double *times,int *Ntimes,double *x,int *delta,int *cause,double
 	       int *CA,int *line,int *detail,double *biid,double *gamiid,int *resample,
 	       double *timepow,int *clusters,int *antclust,double *timepowtest,int *silent,double *convc,
 	       double *weight,double *entry,double *trunkp,int *estimator,int *fixgamma ,int *stratum,
-               int *ordertime,int *robust,double *ssf);
+               int *ordertime,int *robust,double *ssf,double *KMtimes);
 
 
 void LevenbergMarquardt(matrix *S,matrix *SI,vector *U,vector *delta,double *lm,double *step); 
