@@ -15,48 +15,78 @@ void nclusters(int  *npers,int  *clusters, int  *nclust, int  *uniqueclust, int
   mclust[0]=maxclust; 
 }
 
-void clusterindex(int *clusters,int *nclust,int *npers,int *idclust,int *clustsize,int *mednum,int *num)
+void clusterindex(int *clusters,int *nclust,int *npers,int *idclust,int *clustsize,int *mednum,int *num,
+		  int *firstclustid)
 {
   int i;
   if (*mednum==0) {
      for (i=0;i<*npers;i++){
          idclust[(clustsize[clusters[i]])*(*nclust)+clusters[i]]=i; 
          clustsize[clusters[i]]+=1; 
+	 if (clustsize[clusters[i]]==1) firstclustid[clusters[i]]=i; 
       } 
   } else {
     for (i=0;i<*npers;i++){
         idclust[num[i]*(*nclust)+clusters[i]]=i; 
         clustsize[clusters[i]]+=1; 
+        if (clustsize[clusters[i]]==1) firstclustid[clusters[i]]=i; 
+     } 
+  }
+}
+
+void atriskindex(double *start,double *stop,int *id,int *n,double *times,int *ntimes,int *nrisk,int *riskindex)
+{
+  int i,j;
+    for (j=0;j<*ntimes;j++)
+    for (i=0;i<*n;i++)
+    if ((start[i]<times[j]) && (stop[i]>=times[j]))
+    {
+       riskindex[(nrisk[j])*(*ntimes)+j]=id[i]; 
+       nrisk[j]+=1; 
+    } 
+}
+
+void clusterindexdata(int *clusters,int *nclust,int *npers,int *idclust,int *clustsize,int *mednum,
+		int *num,double *data, int *p,double *nydata)
+{
+  int i,j;
+  if (*mednum==0) {
+     for (i=0;i<*npers;i++){
+         idclust[(clustsize[clusters[i]])*(*nclust)+clusters[i]]=i; 
+     for (j=0;j<*p;j++) nydata[(clustsize[clusters[i]]*(*p)+j)*(*nclust)+clusters[i]]=data[(*npers)*j+i]; 
+         clustsize[clusters[i]]+=1; 
+      } 
+  } else {
+    for (i=0;i<*npers;i++){
+        idclust[num[i]*(*nclust)+clusters[i]]=i; 
+        for (j=0;j<*p;j++) nydata[(num[i]*(*p)+j)*(*nclust)+clusters[i]]=data[(*npers)*j+i]; 
+        clustsize[clusters[i]]+=1; 
      } 
   }
 }
 
 /* compute the values of a step function, 
    ie how many of the jumps are smaller or
-   equal to the eval points  */
+   equal to the eval points from prodlim THomas Gerds */
 
-void sindex(int *index, double *jump, double *eval, int *njump, int *neval){
-  int n,nt,i,t;
-
-  n = *njump;
-  
-  nt = *neval;
-
-  index[0] = 0;
-
-  i = 0;
-  
-  for (t=0;t<nt;t++){
-    
-    while(jump[i]<=eval[t]
-	  && i<n)
-      i++;
-
-    index[t] = i;
-  }
+void sindex(int *index, double *jump, double *eval, int *N, int *NT, int *strict){
+int i,t;
+index[0] = 0;
+i = 0;
+if (*strict==0){
+for (t=0;t<*NT;t++){
+	     while(i<*N && jump[i]<=eval[t]) i++;
+		   index[t] = i;
+		       }
+ }
+else{
+ for (t=0;t<*NT;t++){
+	       while(i<*N && jump[i] < eval[t]) i++;
+		     index[t] = i;
+			 }
+   }
 }
 
-
 void Cpred(cum,nx,px,xval,nxval,pred,tminus)
 double *cum,*xval;
 int *nxval,*nx,*px,*tminus,*pred;
